package cn.com.taiji.web.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.domain.Result;
import cn.com.taiji.cql.model.GeoDataSource;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.web.config.EnterpriseConfig;
import cn.com.taiji.web.model.DeviceInfo;
import cn.com.taiji.web.model.StaticsInfo;
import cn.com.taiji.web.service.IEnterpriseService;
import cn.com.taiji.web.vo.DeviceVo;
import cn.com.taiji.web.vo.EnterpriseVo;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.geotools.feature.visitor.CountVisitor;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

@Slf4j
@Service
public class EnterpriseServiceImpl implements IEnterpriseService {

    @Autowired
    private EnterpriseConfig enterpriseConfig;

    @Autowired
    private IECqlService ecqlService;

    @Override
    public PageResult enterpriseQuery(EnterpriseVo enterpriseVo) {

        //数据源
        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.DBTYPE.key, enterpriseConfig.getDbType());

        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put(JDBCDataStoreFactory.USER.key, enterpriseConfig.getDsUserName());
        entDsParams.put(JDBCDataStoreFactory.PASSWD.key, enterpriseConfig.getDsPassword());
        entDsParams.put("tableName", enterpriseConfig.getEnterprise());

        //组装分页参数
        Map<String, Object> pageParams = new HashMap<String, Object>();
        pageParams.put("pageSize",enterpriseVo.getPageSize());
        pageParams.put("pageNumber",enterpriseVo.getPageNumber());

        Map<String, Object> sortParams = new HashMap<String, Object>();
        sortParams.put("id","desc");

        //组转cql查询条件
        StringBuffer filterCql = new StringBuffer();
        if(enterpriseVo.getEntName() !=null && !"".equals(enterpriseVo.getEntName())){

            filterCql.append("(qymc like '%"+enterpriseVo.getEntName()+"%')");

        }

        if(enterpriseVo.getEntType() !=null && !"".equals(enterpriseVo.getEntType())){

            String result = transEntTypeListToSqlString(Arrays.asList(enterpriseVo.getEntType().split(",")));

            if(filterCql.length() >0){

                filterCql.append(" and (qykx in ("+result+"))");
            }else{

                filterCql.append(" (qykx in ("+result+"))");
            }

        }

        System.out.println("查询条件:"+filterCql.toString());

        return ecqlService.cqlQuery4Mysql(entDsParams,filterCql.toString(),sortParams,pageParams);
    }

    public String transEntTypeListToSqlString(List<String> entTypeList) {

        StringBuilder sqlString = new StringBuilder();

        for(String item:entTypeList){

            sqlString.append("'"+item+"',");

        }

        String result = sqlString.toString();

        return result.substring(0,result.lastIndexOf(","));
    }

    @Override
    public Result deviceQuery(DeviceVo deviceVo) {

        //数据源
        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.DBTYPE.key, enterpriseConfig.getDbType());

        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put(JDBCDataStoreFactory.USER.key, enterpriseConfig.getDsUserName());
        entDsParams.put(JDBCDataStoreFactory.PASSWD.key, enterpriseConfig.getDsPassword());
        entDsParams.put("tableName", enterpriseConfig.getDevice());

        //组转cql查询条件
        StringBuffer filterCql = new StringBuffer();

        String wkt = "POINT ("+deviceVo.getLon()+" "+deviceVo.getLat()+")";

        filterCql.append("(DWITHIN(location," + wkt + ", "+deviceVo.getRadius()+", kilometers))");



        PageResult pageResult = ecqlService.cqlQuery4Mysql(entDsParams,filterCql.toString(),null,null);

        List<DeviceInfo> deviceInfoList = new ArrayList<DeviceInfo>();

        if(pageResult.getResp_code() ==0){

            List<Map<String, Object>> dataList = (List<Map<String, Object>>) pageResult.getDatas();

            for(Map<String, Object> dataMap:dataList){

                if(dataMap.containsKey("location")){

                    dataMap.remove("location");

                }

                DeviceInfo deviceInfo = JSONObject.parseObject(JSONObject.toJSONString(dataMap), DeviceInfo.class);
                deviceInfoList.add(deviceInfo);

            }
        }

        System.out.println("查询条件:"+filterCql.toString()+" 记录数:"+deviceInfoList.size());


        return Result.of(deviceInfoList,pageResult.getResp_code(),pageResult.getResp_msg());


    }

    @Override
    public Result staticsQuery() {

        //数据源
        Map<String, Object> entDsParams = new HashMap<String, Object>();
        entDsParams.put(JDBCDataStoreFactory.DATABASE.key, enterpriseConfig.getDsBaseName());
        entDsParams.put(JDBCDataStoreFactory.DBTYPE.key, enterpriseConfig.getDbType());

        entDsParams.put(JDBCDataStoreFactory.HOST.key, enterpriseConfig.getDsIp());
        entDsParams.put(JDBCDataStoreFactory.PORT.key, enterpriseConfig.getDsPort());
        entDsParams.put(JDBCDataStoreFactory.USER.key, enterpriseConfig.getDsUserName());
        entDsParams.put(JDBCDataStoreFactory.PASSWD.key, enterpriseConfig.getDsPassword());
        entDsParams.put("tableName", enterpriseConfig.getEnterprise());

        //计算企业总数

        CountVisitor countVisitor = new CountVisitor();
        ecqlService.cqlAggQuery4Mysql(entDsParams,null,countVisitor);

        //计算零关税自用进口生产设备数量
        CountVisitor deviceCountVisitor = new CountVisitor();
        String cqlDevice = "(qykx ='零关税自用进口生产设备')";
        ecqlService.cqlAggQuery4Mysql(entDsParams,cqlDevice,deviceCountVisitor);

        //计算零关税进口原辅料数量
        CountVisitor materialCountVisitor = new CountVisitor();
        String cqlMaterial = "(qykx ='零关税进口原辅料')";
        ecqlService.cqlAggQuery4Mysql(entDsParams,cqlMaterial,materialCountVisitor);

        //计算加工增值免关税数量
        CountVisitor noTaxCountVisitor = new CountVisitor();
        String cqlNoTax = "(qykx ='加工增值免关税')";
        ecqlService.cqlAggQuery4Mysql(entDsParams,cqlNoTax,noTaxCountVisitor);

        StaticsInfo staticsInfo = new StaticsInfo();
        staticsInfo.setCount(countVisitor.getCount());
        staticsInfo.setDeviceCount(deviceCountVisitor.getCount());
        staticsInfo.setMaterialCount(materialCountVisitor.getCount());
        staticsInfo.setNoTaxCount(noTaxCountVisitor.getCount());

        if(countVisitor.getCount()>0){

            BigDecimal deviceRate = new BigDecimal(deviceCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount())).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);

            staticsInfo.setDeviceRate(deviceRate.stripTrailingZeros());

            BigDecimal materialRate = new BigDecimal(materialCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount())).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);
            staticsInfo.setMaterialRate(materialRate.stripTrailingZeros());

            BigDecimal noTaxRate = new BigDecimal(noTaxCountVisitor.getCount()).divide(new BigDecimal(countVisitor.getCount())).multiply(new BigDecimal(100)).setScale(2, RoundingMode.HALF_UP);

            staticsInfo.setNoTaxRate(noTaxRate.stripTrailingZeros());

        }


        return Result.of(staticsInfo,0,"操作成功！");
    }
}
